package wood.keith.opentools.javad;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.vfs.Url;

/**
 * Class loader for JavaD to load from current JBuilder project class path.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  12 November 2002
 */
public class JBuilderClassLoader extends ClassLoader {

	/**
	 * Look through the current class path for this project for the class.
	 * This may not be the same as the JBuilder class path.
	 *
	 * @param  name  the full name of the class to load
	 * @return  the reference to that class
	 * @throws  ClassNotFoundException  if the class cannot be found
	 */
	protected Class findClass(String name) throws ClassNotFoundException {
		String classLoc = name.replace('.', '/') + ".class";
		// Find the class paths
		Url[] classPaths = ((JBProject)Browser.getActiveBrowser().getActiveProject()).
			getPaths().getFullClassPath();
		for (int index = 0; index < classPaths.length; index++) {
			Url classPath = classPaths[index].getRelativeUrl(classLoc);
			if (classPath.getFileObject().exists()) {
				try {
					// Load the file
					InputStream in =
						new BufferedInputStream(new FileInputStream(classPath.getFile()));
					ByteArrayOutputStream out = new ByteArrayOutputStream();
					byte[] buffer = new byte[1024];
					int count = 0;
					while ((count = in.read(buffer)) > -1) {
						out.write(buffer, 0, count);
					}
					// And define the class
					return defineClass(name, out.toByteArray(), 0, out.size());
				}
				catch (IOException ioe) {
					continue;
				}
			}
		}
		throw new ClassNotFoundException(name);
	}
}